//+------------------------------------------------------------------+
//|                                                  GSMinformer.mq5 |
//|                                         Copyright 2013, Decanium |
//|                                          http://www.decanium.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2013, Decanium"
#property link      "http://www.decanium.com"
#property version   "1.00"
//---
#include <modem.mqh>
//--- input parameters
input string         str00="=== COM port settings ====";
input ComPortList    inp_com_port_index=COM3;    // Selecting the COM port
input BaudRateList   inp_com_baudrate=_9600bps;  // Data rate
//---
input string         str01="======= Modem ==============";
input int            inp_refr_period=3;          // Modem query period, sec
input int            inp_ussd_request_tout=20;   // Timeout for response to a USSD request, sec
input string         inp_sms_service_center="";  // SMS service center number
//---
input string         str02="=== Balance ======";
input int            inp_refr_bal_period=12;     // Query period, hr
input string         inp_ussd_get_balance="";    // Balance USSD request
input string         inp_ussd_bal_suffix="";     // Balance suffix
input string         inp_ussd_exp_prefix="";     // Prefix of the number expiration date
//---
input string         str03="= Number of package SMS ==";
input int            inp_refr_smscnt_period=6;   // Query period, hr
input string         inp_ussd_get_sms_cnt="";    // USSD request for the package service status
input string         inp_ussd_sms_suffix="";     // SMS counter suffix
input int            inp_free_sms_daily=0;       // Daily SMS limit
//---
input string         str04="============================";
input string         inp_admin_number="";        // Administrator's phone number
input int            inp_conn_hyst=6;            // Hysteresis, 10 sec
input int            inp_sms_max_old=600;        // SMS command expiration, sec

bool sms_mem_full=false;
//+------------------------------------------------------------------+
//| IncomingSMS                                                      |
//| Called when a new SMS message is received                        |
//+------------------------------------------------------------------+
void IncomingSMS(INCOMING_SMS_STR &sms)
  {
   string str,strtmp;
//--- number from which the last received SMS message was sent:
   ObjectSetString(0,"str18",OBJPROP_TEXT,"SMS number:      "+sms.sender);
//--- date and time of sending the last received SMS message:
   str=TimeToString(sms.scts.time,TIME_DATE|TIME_SECONDS);
   ObjectSetString(0,"str19",OBJPROP_TEXT,"SMS date/time:   "+str);
//--- text of the last received SMS message:
   strtmp=StringSubstr(sms.text,0,32); str=" ";
   if(strtmp!="") str=strtmp;
   ObjectSetString(0,"str20",OBJPROP_TEXT,str);
   strtmp=StringSubstr(sms.text,32,32); str=" ";
   if(strtmp!="") str=strtmp;
   ObjectSetString(0,"str21",OBJPROP_TEXT,str);
   strtmp=StringSubstr(sms.text,64,32); str=" ";
   if(strtmp!="") str=strtmp;
   ObjectSetString(0,"str22",OBJPROP_TEXT,str);
   strtmp=StringSubstr(sms.text,96,32); str=" ";
   if(strtmp!="") str=strtmp;
   ObjectSetString(0,"str23",OBJPROP_TEXT,str);
   strtmp=StringSubstr(sms.text,128,32); str=" ";
   if(strtmp!="") str=strtmp;
   ObjectSetString(0,"str24",OBJPROP_TEXT,str);
//
   if(sms.sender==inp_admin_number)
     {
      Print("SMS from the administrator");
      datetime t=TimeLocal();
      if(t-sms.scts.time<=inp_sms_max_old)
        {
         //Print("SMS delivered in "+DoubleToString(t-sms.scts.time,0)+"seconds");
         //--- check if it is a command
         string cmdstr=sms.text;
         StringToUpper(cmdstr);
         //Print(cmdstr);
         int pos= StringFind(cmdstr,"CLOSE",0);
         cmdstr = StringSubstr(cmdstr, pos+6, 6);
         if(pos>=0)
           {//--- command. send it for processing
            ClosePositions(cmdstr);
           }
        }
      else
         Print("The SMS command has expired");
     }
  }
//+------------------------------------------------------------------+
//| ClosePositions                                                   |
//| Closing the specified positions                                  |
//+------------------------------------------------------------------+
uint ClosePositions(string sstr)
  {
   bool all=false;
   if(StringFind(sstr,"ALL",0)>=0)
      all=true;
//Print("Close "+sstr);
   uint res = 0;
   for(int i=0;i<PositionsTotal();i++)
     {
      string symbol=PositionGetSymbol(i);
      if(all==true || sstr==symbol)
        {
         if(PositionSelect(symbol)==true)
           {
            long pos_type;
            double pos_vol;
            if(PositionGetInteger(POSITION_TYPE,pos_type)==true)
              {
               if(PositionGetDouble(POSITION_VOLUME,pos_vol)==true)
                 {
                  if(OrderClose(symbol,(ENUM_POSITION_TYPE)pos_type,pos_vol)==true)
                     res|=0x01;
                  else
                     res|=0x02;
                 }
              }
           }
        }
     }
   return(res);
  }
//+------------------------------------------------------------------+
//| OrderClose                                                       |
//+------------------------------------------------------------------+
bool OrderClose(string symbol,ENUM_POSITION_TYPE pos_type,double vol)
  {
//
//Print("CLOSE");
//
   MqlTick last_tick;
   MqlTradeRequest request;
   MqlTradeResult result;
   double price=0;
//
   ZeroMemory(request);
   ZeroMemory(result);
//
   if(SymbolInfoTick(Symbol(),last_tick))
     {
      price=last_tick.bid;
     }
   else
     {
      Print("Error when getting current prices");
      return(false);
     }
//   
   if(pos_type==POSITION_TYPE_BUY)
     {
      //--- closing a BUY position - SELL
      request.type=ORDER_TYPE_SELL;
     }
   else if(pos_type==POSITION_TYPE_SELL)
     {
      //--- closing a SELL position - BUY
      request.type=ORDER_TYPE_BUY;
     }
   else
      return(false);
//
   request.price=NormalizeDouble(price,_Digits);
   request.deviation=20;
   request.action = TRADE_ACTION_DEAL;
   request.symbol = symbol;
   request.volume = NormalizeDouble(vol, 2);
   if(request.volume==0)
      return(false);
   request.type_filling=ORDER_FILLING_FOK;
//
   if(OrderSend(request,result)==true)
     {
      if(result.retcode==TRADE_RETCODE_DONE || result.retcode==TRADE_RETCODE_DONE_PARTIAL)
        {
         Print("Order executed successfully");
         return(true);
        }
     }
   else
     {
      Print("Order parameter error: ",GetLastError(),", Trade server return code: ",result.retcode);
      return(false);
     }
//
   return(false);
  }
//+------------------------------------------------------------------+
//| SMSMemoryFull                                                    |
//| SMS memory is full                                               |
//+------------------------------------------------------------------+
void SMSMemoryFull(int n)
  {
   sms_mem_full=true;
   for(int i=0; i<n; i++)
     {//--- delete all SMS messages
      if(DelSMSbyIndex(i)==false)
         break;
      else
         sms_mem_full=false;
     }
  }
//+------------------------------------------------------------------+
//| IncomingCall                                                     |
//| Called upon receiving an incoming call                           |
//+------------------------------------------------------------------+
void IncomingCall(string number)
  {//
//Print("Incoming Call: "+number);
   bool result;
   if(number==inp_admin_number)
     {
      Print("Administrator's phone number. Sending SMS.");
      //
      string mob_bal="";
      //---mobile account balance
      if(modem.bal!=-10000)
         mob_bal="\n(m.bal="+DoubleToString(modem.bal,2)+")";
      result=SendSMS(inp_admin_number,//"Server: " +AccountInfoString(ACCOUNT_SERVER)
                     "Account: "+DoubleToString(AccountInfoInteger(ACCOUNT_LOGIN),0)
                     +"\nProfit: "+DoubleToString(AccountInfoDouble(ACCOUNT_PROFIT),2)
                     +"\nEquity: "+DoubleToString(AccountInfoDouble(ACCOUNT_EQUITY),2)
                     +"\nPositions: "+DoubleToString(PositionsTotal(),0)
                     +mob_bal
                     ,false);
      if(result==true)
         Print("SMS sent successfully");
      else
         Print("Error when sending SMS");
     }
   else
      Print("Unauthorized number ("+number+")");
//--- update the number of the last incoming call:
   ObjectSetString(0,"str16",OBJPROP_TEXT,"Incoming:        "+number);
  }
//+------------------------------------------------------------------+
//| ModemChState                                                     |
//+------------------------------------------------------------------+
void ModemChState()
  {
   static bool init_ok=false;
//Print("Modem status changed");
   if(modem.init_ok==true && init_ok==false)
     {
      Print("Modem initialized successfully");
      init_ok=true;
     }
//---
   RefreshTab();
  }
//+------------------------------------------------------------------+
//| TextXY                                                           |
//+------------------------------------------------------------------+
void TextXY(string ObjName,string Text,int x,int y,color TextColor)
  {
//--- display the text string
   ObjectDelete(0,ObjName);
   ObjectCreate(0,ObjName,OBJ_LABEL,0,0,0,0,0);
   ObjectSetInteger(0,ObjName,OBJPROP_XDISTANCE,x);
   ObjectSetInteger(0,ObjName,OBJPROP_YDISTANCE,y);
   ObjectSetInteger(0,ObjName,OBJPROP_COLOR,TextColor);
   ObjectSetInteger(0,ObjName,OBJPROP_FONTSIZE,9);
   ObjectSetString(0,ObjName,OBJPROP_FONT,"Courier New");
   ObjectSetString(0,ObjName,OBJPROP_TEXT,Text);
  }
//+------------------------------------------------------------------+
//| DrawTab                                                          |
//| Drawing the table of modem parameters                            |
//+------------------------------------------------------------------+
void DrawTab()
  {
   int   x=20, //horizontal indent 
   y = 20,     //vertical indent
   dy = 15;    //step along the Y-axis
//--- draw the background
   ObjectDelete(0,"bgnd000");
   ObjectCreate(0,"bgnd000",OBJ_RECTANGLE_LABEL,0,0,0,0,0);
   ObjectSetInteger(0,"bgnd000",OBJPROP_XDISTANCE,x-10);
   ObjectSetInteger(0,"bgnd000",OBJPROP_YDISTANCE,y-5);
   ObjectSetInteger(0,"bgnd000",OBJPROP_XSIZE,270);
   ObjectSetInteger(0,"bgnd000",OBJPROP_YSIZE,420);
   ObjectSetInteger(0,"bgnd000",OBJPROP_BGCOLOR,clrBlack);
//--- port parameters
   TextXY("str0",  "Port:            ", x, y, clrWhite); y+=dy;
   TextXY("str1",  "Speed:           ", x, y, clrWhite); y+=dy;
   TextXY("str2",  "Rx:              ", x, y, clrWhite); y+=dy;
   TextXY("str3",  "Tx:              ", x, y, clrWhite); y+=dy;
   TextXY("str4",  "Err:             ", x, y, clrWhite); y+=(dy*3)/2;
//--- modem parameters
   TextXY("str5",  "Modem:           ", x, y, clrWhite); y+=dy;
   TextXY("str6",  "SIM:             ", x, y, clrWhite); y+=dy;
   TextXY("str7",  "NET:             ", x, y, clrWhite); y+=dy;
   TextXY("str8",  "Operator:        ", x, y, clrWhite); y+=dy;
   TextXY("str9",  "SMSC:            ", x, y, clrWhite); y+=dy;
   TextXY("str10", "RSSI:            ", x, y, clrWhite); y+=dy;
   TextXY("str11", "Bat:             ", x, y, clrWhite); y+=dy;
   TextXY("str12", "Modem status:    ", x, y, clrWhite); y+=(dy*3)/2;
//--- mobile account balance
   TextXY("str13", "Balance:         ", x, y, clrWhite); y+=dy;
   TextXY("str14", "Expiration date: ", x, y, clrWhite); y+=dy;
   TextXY("str15", "Free SMS:        ", x, y, clrWhite); y+=(dy*3)/2;
//--- number of the last incoming call
   TextXY("str16","Incoming:        ",x,y,clrWhite); y+=(dy*3)/2;
//--- parameters of the last received SMS message
   TextXY("str17", "SMS mem full:    ", x, y, clrWhite); y+=dy;
   TextXY("str18", "SMS number:      ", x, y, clrWhite); y+=dy;
   TextXY("str19", "SMS date/time:   ", x, y, clrWhite); y+=dy;
//--- text of the last received SMS message
   TextXY("str20", "                 ", x, y, clrGray); y+=dy;
   TextXY("str21", "                 ", x, y, clrGray); y+=dy;
   TextXY("str22", "                 ", x, y, clrGray); y+=dy;
   TextXY("str23", "                 ", x, y, clrGray); y+=dy;
   TextXY("str24", "                 ", x, y, clrGray); y+=dy;
//---
   ChartRedraw(0);
  }
//+------------------------------------------------------------------+
//| RefreshTab()                                                     |
//| Refreshing values in the table                                   |
//+------------------------------------------------------------------+
void RefreshTab()
  {
   string str;
//--- COM port index:
   str="COM"+DoubleToString(PortID+1,0);
   ObjectSetString(0,"str0",OBJPROP_TEXT,"Port:            "+str);
//--- data rate:
   str=DoubleToString(inp_com_baudrate,0)+" bps";
   ObjectSetString(0,"str1",OBJPROP_TEXT,"Speed:           "+str);
//--- number of bytes received:
   str=DoubleToString(rx_cnt,0)+" bytes";
   ObjectSetString(0,"str2",OBJPROP_TEXT,"Rx:              "+str);
//--- number of bytes transmitted:
   str=DoubleToString(tx_cnt,0)+" bytes";
   ObjectSetString(0,"str3",OBJPROP_TEXT,"Tx:              "+str);
//--- number of port errors:
   str=DoubleToString(tx_err,0);
   ObjectSetString(0,"str4",OBJPROP_TEXT,"Err:             "+str);
//--- modem manufacturer and model:
   str=modem.manufacturer+" "+modem.device;
   ObjectSetString(0,"str5",OBJPROP_TEXT,"Modem:           "+str);
//--- SIM card status:
   string sim_stat_str[2]={"Error","Ok"};
   if(modem.sim_stat==-1)
      str="n/a";
   else
      str=sim_stat_str[modem.sim_stat];
   ObjectSetString(0,"str6",OBJPROP_TEXT,"SIM:             "+str);
//--- network registration:
   string net_reg_str[6]={"No","Ok","Search...","Restricted","Unknown","Roaming"};
   if(modem.net_reg==-1)
      str="n/a";
   else
      str=net_reg_str[modem.net_reg];
   ObjectSetString(0,"str7",OBJPROP_TEXT,"NET:             "+str);
//--- name of mobile operator:
   ObjectSetString(0,"str8",OBJPROP_TEXT,"Operator:        "+modem.op);
//--- SMS service center number
   ObjectSetString(0,"str9",OBJPROP_TEXT,"SMSC:            "+modem.sms_sca);
//--- signal level:
   if(modem.rssi==-1)
      str="n/a";
   else
      str=rssi_to_str(modem.rssi);
   ObjectSetString(0,"str10",OBJPROP_TEXT,"RSSI:            "+str);
//--- battery status (applicable to phones):
   string bat_stats_str[4]={"Ok, ","Ok, ","No","Err"};
   if(modem.bat_stat==-1)
      str="n/a";
   else
      str=bat_stats_str[modem.bat_stat];
   if(modem.bat_stat==0 || modem.bat_stat==1)
      str+=DoubleToString(modem.bat_charge,0)+"%";
   ObjectSetString(0,"str11",OBJPROP_TEXT,"Bat:             "+str);
//--- modem status:
   string modem_stat_str[5]={"Ready","Err","Err","Incoming call","Active call"};
   if(modem.status==-1)
      str="init...";
   else
     {
      if(modem.status>4 || modem.status<0)
         Print("Unknown modem status: "+DoubleToString(modem.status,0));
      else
         str=modem_stat_str[modem.status];
     }
   ObjectSetString(0,"str12",OBJPROP_TEXT,"Modem status:    "+str);
//--- mobile account balance:
   if(modem.bal==-10000)
      str="n/a";
   else
      str=DoubleToString(modem.bal,2)+" "+inp_ussd_bal_suffix;
   ObjectSetString(0,"str13",OBJPROP_TEXT,"Balance:         "+str);
//--- mobile number expiration date:
   ObjectSetString(0,"str14",OBJPROP_TEXT,"Expiration date: "+modem.exp_date);
//--- package SMS available:
   if(modem.sms_free<0)
      str="n/a";
   else
      str=DoubleToString(modem.sms_free,0);
   ObjectSetString(0,"str15",OBJPROP_TEXT,"Free SMS:        "+str);
//--- SMS memory full:
   if(sms_mem_full==true)
      str="Yes";
   else
      str="No";
   ObjectSetString(0,"str17",OBJPROP_TEXT,"SMS mem full:    "+str);
//---
   ChartRedraw(0);
  }
//+------------------------------------------------------------------+
//| DelTab                                                           |
//| Deleting the table                                               |
//+------------------------------------------------------------------+
void DelTab()
  {
   for(int i=0; i<25; i++)
      ObjectDelete(0,"str"+DoubleToString(i,0));
   ObjectDelete(0,"bgnd000");
  }
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- COM port initialization
   if(InitComPort()==false)
     {
      Print("Error when initializing the COM"+DoubleToString(inp_com_port_index+1,0)+" port");
      return(INIT_FAILED);
     }
//---
   DrawTab();
//--- modem initialization
   InitModem();
//--- setting the timer 
   EventSetTimer(1);//1 second interval
//---
   RefreshTab();
//---
   conn_mon_init();
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- destroy timer
   EventKillTimer();
   DeinitComPort();
   DelTab();
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
  }
//
//+------------------------------------------------------------------+
//| POS_STR                                                          |
//+------------------------------------------------------------------+
struct POS_STR
  {
   bool              new_event;
   string            symbol;
   ulong             deal;
   ENUM_DEAL_TYPE    deal_type;
   double            volume;
   double            price;
  };

#define POS_BUF_LEN  3

POS_STR ps[POS_BUF_LEN];
//  

//+------------------------------------------------------------------+
//| Structure of monitoring connection with the terminal             |
//+------------------------------------------------------------------+
struct CONN_MON_STR
  {
   bool              new_state;    //flag of change in the connection status
   bool              connected;    //connection status
   int               conn_cnt;     //counter of successful connection queries
   int               disconn_cnt;  //counter of unsuccessful connection queries
   datetime          conn_time;    //time of established connection
   datetime          disconn_time; //time of lost connection
  };

CONN_MON_STR cm;//structure of connection monitoring  
//+------------------------------------------------------------------+
//| conn_mon_init                                                    |
//+------------------------------------------------------------------+
void conn_mon_init()
  {
   cm.connected= true;
   cm.conn_cnt = inp_conn_hyst;
   cm.disconn_cnt=0;
   cm.conn_time = TimeLocal();
   cm.new_state = false;
  }
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
//---
   ModemTimerProc();//call every second
                    //
   static int s10=0;//pre-divider by 10 seconds
   static datetime conn_time;
   static datetime disconn_time;
   if(++s10>=10)
     {//once every 10 seconds
      s10=0;
      //
      if((bool)TerminalInfoInteger(TERMINAL_CONNECTED)==true)
        {
         if(cm.conn_cnt==0)             //first successful query in the sequence
            conn_time = TimeLocal();    //save the time
         if(cm.conn_cnt<inp_conn_hyst)
           {
            if(++cm.conn_cnt>=inp_conn_hyst)
              {
               //--- connection has been stabilized
               if(cm.connected==false)
                 {
                  //--- if there was a long-standing connection loss prior to that
                  cm.connected = true;
                  cm.new_state = true;
                  cm.conn_time = conn_time;
                 }
              }
           }
         //Print("Connected");
         cm.disconn_cnt=0;
        }
      else
        {
         if(cm.disconn_cnt==0)          //first unsuccessful query in the sequence
            disconn_time = TimeLocal(); //save the time
         if(cm.disconn_cnt<inp_conn_hyst)
           {
            //--- long-standing connection loss
            if(++cm.disconn_cnt>=inp_conn_hyst)
              {
               //--- if the connection was stable prior to that 
               if(cm.connected==true)
                 {
                  cm.connected = false;
                  cm.new_state = true;
                  cm.disconn_time=disconn_time;
                 }
              }
           }
         //Print("Disconnected");   
         cm.conn_cnt=0;
        }
      //
     }
//
//--- connection status changed
   if(cm.new_state==true)
     {
      if(cm.connected==true)
        {//--- connection is available
         string str="Connected "+TimeToString(cm.conn_time,TIME_DATE|TIME_SECONDS);
         if(cm.disconn_time!=0)
            str+=", offline: "+dTimeToString((ulong)(cm.conn_time-cm.disconn_time));
         Print(str);
         SendSMS(inp_admin_number,str,false);//sending message
        }
      else
        {//--- no connection
         string str="Disconnected "+TimeToString(cm.disconn_time,TIME_DATE|TIME_SECONDS);
         if(cm.conn_time!=0)
            str+=", online: "+dTimeToString((ulong)(cm.disconn_time-cm.conn_time));
         Print(str);
         SendSMS(inp_admin_number,str,false);//sending message
        }
      cm.new_state=false;
     }
//--- processing of the opening/closing of positions
   string posstr="";
   for(int i=0;i<POS_BUF_LEN;i++)
     {
      if(ps[i].new_event==true)
        {
         string str;
         if(ps[i].deal_type==DEAL_TYPE_BUY)
            str+="Buy ";
         else if(ps[i].deal_type==DEAL_TYPE_SELL)
            str+="Sell ";
         str+=DoubleToString(ps[i].volume,2)+" "+ps[i].symbol;
         int digits=(int)SymbolInfoInteger(ps[i].symbol,SYMBOL_DIGITS);
         str+=", price="+DoubleToString(ps[i].price,digits);
         //
         long deal_entry;
         HistorySelect(TimeCurrent()-3600,TimeCurrent());//retrieve the history for the last hour
         if(HistoryDealGetInteger(ps[i].deal,DEAL_ENTRY,deal_entry)==true)
           {
            if(((ENUM_DEAL_ENTRY)deal_entry)==DEAL_ENTRY_IN)
               str+=", entry: in";
            else if(((ENUM_DEAL_ENTRY)deal_entry)==DEAL_ENTRY_OUT)
              {
               str+=", entry: out";
               double profit;
               if(HistoryDealGetDouble(ps[i].deal,DEAL_PROFIT,profit)==true)
                 {
                  str+=", profit = "+DoubleToString(profit,2);
                 }
              }
           }
         posstr+=str+"\r\n";
         ps[i].new_event=false;
        }
     }
   if(posstr!="")
     {
      Print(posstr+"pos: "+DoubleToString(PositionsTotal(),0));
      SendSMS(inp_admin_number,posstr+"pos: "+DoubleToString(PositionsTotal(),0),false);
     }
//
  }
//+------------------------------------------------------------------+
//| dTimeToString                                                    |
//+------------------------------------------------------------------+
string dTimeToString(ulong sec)
  {
   string str;
   uint days=(uint)(sec/86400);
   if(days>0)
     {
      str+= DoubleToString(days,0)+" days, ";
      sec-= days*86400;
     }
   uint hour=(uint)(sec/3600);
   if(hour<10) str+="0";
   str+= DoubleToString(hour,0)+":";
   sec-= hour*3600;
   uint min=(uint)(sec/60);
   if(min<10) str+="0";
   str+= DoubleToString(min,0)+":";
   sec-= min*60;
   if(sec<10) str+="0";
   str+=DoubleToString(sec,0);
//
   return(str);
  }
//+------------------------------------------------------------------+
//| TradeTransaction function                                        |
//+------------------------------------------------------------------+
void OnTradeTransaction(const MqlTradeTransaction &trans,
                        const MqlTradeRequest &request,
                        const MqlTradeResult &result)
  {
//---
   if(trans.type==TRADE_TRANSACTION_DEAL_ADD)
     {
      if(trans.deal_type==DEAL_TYPE_BUY ||
         trans.deal_type==DEAL_TYPE_SELL)
        {
         int i;
         for(i=0;i<POS_BUF_LEN;i++)
           {
            if(ps[i].new_event==false)
               break;
           }
         if(i<POS_BUF_LEN)
           {
            ps[i].new_event = true;
            ps[i].deal_type = trans.deal_type;
            ps[i].symbol = trans.symbol;
            ps[i].volume = trans.volume;
            ps[i].price= trans.price;
            ps[i].deal = trans.deal;
           }
        }
     }
//   
  }
//+------------------------------------------------------------------+
